"""
Global topics on the message bus.
"""

AUTH = 'ETT-AUTH'
CERTS = 'ETT-CERTS'
DELEGATES = 'ETT-DELEGATES'
KEYS = 'ETT-KEYS'
MEASUREMENTS = 'ETT-MEASUREMENTS'
MEASUREMENTS_COMMANDS = 'ETT-MEASUREMENTS-COMMANDS'
METERINGPOINTS = 'ETT-METERINGPOINTS'
METERINGPOINTS_COMMANDS = 'ETT-METERINGPOINTS_COMMANDS'
TECHNOLOGIES = 'ETT-TECHNOLOGIES'
USERS = 'ETT-USERS'
